//main.cpp BEGIN
#include "PN.h"
#include <cmath>
#include <cstdlib>
#include <ctime>
#include <iostream>
#include <random>
#include <SDL.h>
#undef main

#define TEST 400

using namespace std;

void SetPixel(SDL_Surface* surface, int x, int y, Uint32 pixel) //функция установки пикселя на поверхность, принимает название поверхности, координаты х и у, и цвет в формате юинт32
{
    Uint32 *p = (Uint32 *) surface->pixels + y*surface->w+x;
    *p = pixel;
}

int main()
{
    unsigned int width=TEST, height=TEST; //размеры нашего окна

    SDL_Window *window= NULL; //пустое окно
    SDL_Renderer *renderer = NULL; //пустой рендерер
    SDL_Surface *image = NULL; //пустая поверхность
    SDL_Texture *imageTexture = NULL; //пустая текстура

    if(SDL_Init(SDL_INIT_VIDEO)<0)
	{ 
		//инициализируем видеодрайвер или выкидываем ошибку
        cout<<"Ошибка инициализации видео"<<endl;
        return 1;
    }

    window = SDL_CreateWindow("Perlin", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, width, height, 0); //создаем окно с ранее заданными размерами по центру или кидаем ошибку
    if(!window) 
	{
        cout<<"SDL не создало окно"<<endl;
        return 2;
    }

    renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_SOFTWARE); //создаем рендерер для нашего окна или кидаем ошибку
    if(!renderer) 
	{
        cout<<"SDL не получила доступа к рендереру"<<endl;
        return 3;
    }

    Uint32 rmask, gmask, bmask, amask; //создаем пиксель с цветами в формате RGBA
    #if SDL_BYTEORDER == SDL_BIG_ENDIAN //здесь описывается маска пикселей для разных архитектур
        rmask = 0xff000000;
        gmask = 0x00ff0000;
        bmask = 0x0000ff00;
        amask = 0x000000ff;
    #else
        rmask = 0x000000ff;
        gmask = 0x0000ff00;
        bmask = 0x00ff0000;
        amask = 0xff000000;
    #endif

    image = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, 32, rmask, gmask, bmask,amask); //создаем поверхность под наши пиксели
    if(!image) 
	{
        cout << "Все плохо с поверхностью :(" <<endl;
        return 4;
    }
    unsigned int PSeed=0; //зернышко для рандома

    Uint8 colourByte; //упрощаем пиксель до юинта (у нас же ЧБ изображение)
    bool quit = false; //бул для выхода
    SDL_Event event; //событие для обработки нажатий

    PN pn(PSeed); //создаем объект шума Перлина по зерну
    int ax=0, ay=0, e=0; //начальные значения для формулы шума
	while (!quit)
	{ //пока не ESC или не закрыли окно

		for (unsigned int i = 0; i < height; ++i) //циклом высчитываем шум и заполняем поверхность
			for (unsigned int j = 0; j < width; ++j)
			{
				double x = (double)j / ((double)width);
				double y = (double)i / ((double)height);
				double n = pn.PNoise(ax*x, ay*y, e);
				colourByte = Uint8(n * 0xff);
				SetPixel(image, j, i, SDL_MapRGB(image->format, colourByte, colourByte, colourByte));
			}
		SDL_RenderClear(renderer); //очищаем буффер рендерера и пихаем в него новую текстуру
		imageTexture = SDL_CreateTextureFromSurface(renderer, image); //создаем изображение (которое пойдет на экран)
		SDL_RenderCopy(renderer, imageTexture, NULL, NULL);
		SDL_RenderPresent(renderer); //проверяем все ли хорошо

		if (!imageTexture)
		{
			cout << "Не создалась поверхность из текстуры" << endl;
			return 5;
		}

		cout << " ax: " << ax << " ay: " << ay << " e: " << e << endl; //выводим в консоль текущие коэффициенты
		cout << "New ax, ay, e (spaced): ";
		cin >> ax >> ay >> e;

		//этот участок кода скорее всего не понадобится,
		//поскольку выход можно выполнить просто закрыв консоль
		//да и окно отвечать не будет, но пусть
		while (SDL_PollEvent(&event)) { //ждем дальнейших указаний:
			switch (event.type)
			{
			case SDL_QUIT: //закрыли окно -> выход
				quit = true;
				break;
			}
		}
	}
	//выход:
    SDL_FreeSurface(image); //очищаем поверхность
    SDL_DestroyTexture(imageTexture); //уничтожаем текстуру
    SDL_DestroyRenderer(renderer); //и рендерер
    SDL_DestroyWindow(window); // и окно
    SDL_Quit(); //выключаем библиотеку SDL
    return 0; //выход из программы
}
//main.cpp END